# Updated 2021 July 06

#### CODE PREAMBLE AND DESCRIPTION ####

## Code preamble: prepare workspace and load R packages

rm(list = ls())     # Clear workspace
library(data.table) # R package for using data.tables as augmented data.frames
library(readxl)     # R package to read in Excel spreadsheet files
library(lwgeom)     # R package to work with geographic shapefiles (with "sf")
library(sf)         # R package to read in and work with geographic shapefiles
library(zoo)        # R package that has some date indexing functions etc.
library(haven)      # R package that allows working with Stata .dta files
datadir = "~/Dropbox/Shared/SchoolsCovid/data" # SET THIS: top-level data directory
setwd(datadir)

## Data and programming notes:
#
# 1. Running this code requires about 14 GB of RAM.
#
# 2. This code forms a balanced panel of counties:
#    - Some of the columns are "fixed", such as 2019 county population and initial 2020-21 school policy
#    - Some of the columns vary at the daily level, such as county COVID-19 cases and deaths
#    - Some of the columns vary at the monthly level, such as county employment measurements
#
# 3. The time identifiers are date, weekly date (starting weeks with Sunday), and monthly date
#
# 4. Monthly values such as employment are assigned to the first date of each month in the panel
#
# 5. The employment data in the panel starts with January 2019 and ends with April 2021
#
# 6. The other data in the panel starts with 2020-March-01 and ends with 2021-April-30
#
# 7. For each county, assigning school policy using the largest district by enrollment within the county
#

## Data sources:
#
# [A] Downloaded automatically
#
#  1) County COVID-19 cases and deaths by date from the New York Times at 
#     https://github.com/nytimes/covid-19-data
#
#  2) County employment measurements by month from the Bureau of Labor Statistics at 
#     https://www.bls.gov/lau/lausad.htm
#
#  3) County community mobility indexes by date from Google Community Mobility Reports at 
#     https://www.google.com/covid19/mobility/
#
#  4) County boundaries shape file from the Census Bureau at 
#     https://www.census.gov/geographies/mapping-files/time-series/geo/tiger-line-file.2019.html
#     The school district and county boundaries shape files are used to merge districts to counties
#     These data are also used to determine county coordinates (latitude and longitude)
#
#  5) School district boundaries shape file from National Center for Education Statistics at 
#     https://nces.ed.gov/programs/edge/Geographic/DistrictBoundaries
#
#  6) County and state COVID-19 policy ordinances from Department of Health and Human Services at 
#     https://healthdata.gov/dataset/COVID-19-State-and-County-Policy-Orders/gyqz-9u7n
#     Also downloads a crosswalk of state abbreviations to county FIPS codes from Kieran Healy at
#     https://github.com/kjhealy/fips-codes
#
#  7) County population estimates from the Census Bureau at 
#     https://www.census.gov/data/tables/time-series/demo/popest/2010s-counties-total.html
#
#  8) County Gross Domestic Product for 2019 from the Bureau of Economic Analysis at
#     https://apps.bea.gov/regional/downloadzip.cfm
#
#  9) County Small Area Income and Poverty Estimates (SAIPE) for 2019
#     https://www.census.gov/data/datasets/2019/demo/saipe/2019-state-and-county.html
#
# [B] Downloaded manually
#
# 10) School district policies for the 2020-2021 school year from MCH Strategic Data at 
#     https://www.mchdata.com/covid19/schoolclosings (account required)
#     MCH Strategic Data overwrote the data with updates as schools changed policies and more data was available
#     This code uses the version of school district policies data provided on of 2020 November 02
#
# 11) County COVID-19 test volume by date from the Carnegie Mellon University Delphi Group at 
#     https://delphi.cmu.edu/covidcast/export/
#
# 12) County demographic labor force participation from the American Community Survey 2019 5-year from the Census Bureau at
#     https://data.census.gov/
#     Using selected labor force participation data from table S2301
#
# 13) County demographic characteristics from the American Community Survey 2019 5-year from the Census Bureau at
#     https://data.census.gov/
#     Using selected demographic data from table DP05 
#
# 14) County 2019 health care characteristics from the Centers for Medicare and Medicaid Services at
#     https://data.cms.gov/mapping-medicare-disparities
#     Using 2019 county influenza vaccine rates
#
# 15) County 2019 total and ICU hospital bed counts from ArcGIS (ESRI) at
#     https://nation.maps.arcgis.com/home/item.html?id=58c01cc29cad449baa356049f4d53388
#

#### (DOWN)LOAD THE DATA COMPONENTS THAT CAN BE DOWNLOADED AUTOMATICALLY ####

setwd(file.path(datadir, "downloaded_automatically"))

# 1) County COVID-19 cases and deaths by date

if(!file.exists(file.path(datadir, "downloaded_automatically", "covid-us-counties.csv"))) {
  download.file("https://raw.githubusercontent.com/nytimes/covid-19-data/master/us-counties.csv", 
                file.path(datadir, "downloaded_automatically", "covid-us-counties.csv"))}
county.covid <- fread(file.path(datadir, "downloaded_automatically", "covid-us-counties.csv"))
county.covid[, date := as.Date(date, format = "%Y-%m-%d")]

# Correct for a few missing FIPS codes:
county.covid[is.na(fips) & state == "Missouri" & county == "Joplin", fips := 29097]
county.covid[is.na(fips) & state == "Missouri" & county == "Kansas City", fips := 29095]
county.covid[fips == 29095, county := "Jackson"]
county.covid[fips == 29097, county := "Jasper"]
county.covid[county == "New York City", fips := 36061]
county.covid <- county.covid[!is.na(fips) & fips < 57000, .(cases = sum(cases), deaths = sum(deaths)), by = .(fips, date)]
county.covid[, fips := sprintf("%05d", fips)]
setnames(county.covid, "cases", "cumulative_cases")
setnames(county.covid, "deaths", "cumulative_deaths")

# 2) County employment measurements by month

if(!file.exists(file.path(datadir, "downloaded_automatically", "la.data.64.County.txt"))) {
  download.file("https://download.bls.gov/pub/time.series/la/la.data.64.County", 
                file.path(datadir, "downloaded_automatically", "la.data.64.County.txt"), method = "curl")}
county.employment <- fread(file.path(datadir, "downloaded_automatically", "la.data.64.County.txt"))
county.employment <- county.employment[period != "M13" & value != "-"]
county.employment[, fips := substr(series_id, 6, 10)]
county.employment[, monthdate := as.Date(paste(year, substr(period, 2, 3), "01", sep = "-"))]
county.employment[substr(series_id, 19, 20) == "03", unemployment_rate := as.numeric(value)]
county.employment[substr(series_id, 19, 20) == "04", unemployment := as.numeric(value)]
county.employment[substr(series_id, 19, 20) == "05", employment := as.numeric(value)]
county.employment[substr(series_id, 19, 20) == "06", labor_force := as.numeric(value)]
county.employment <- county.employment[, lapply(.SD, max, na.rm = TRUE), by = .(fips, monthdate), 
                                       .SDcols = c("unemployment_rate", "unemployment", "employment", "labor_force")]

# 3) County community mobility indexes by date 

if(!file.exists("Region_Mobility_Report_CSVs.zip")) {
  download.file("https://www.gstatic.com/covid19/mobility/Region_Mobility_Report_CSVs.zip", "Region_Mobility_Report_CSVs.zip")}
unzip("Region_Mobility_Report_CSVs.zip", files = c("2021_US_Region_Mobility_Report.csv", "2020_US_Region_Mobility_Report.csv"))
county.mobility <- rbind(fread("2020_US_Region_Mobility_Report.csv")[sub_region_1 != "" & sub_region_2 != ""],
                  fread("2021_US_Region_Mobility_Report.csv")[sub_region_1 != "" & sub_region_2 != ""])
unlink(c("2021_US_Region_Mobility_Report.csv", "2020_US_Region_Mobility_Report.csv"))

setnames(county.mobility, "census_fips_code", "fips")
county.mobility[, date := as.Date(date, format = "%Y-%m-%d")]
county.mobility[, fips := sprintf("%05d", fips)]
county.mobility[, moveidx_retail_recreation := as.integer(retail_and_recreation_percent_change_from_baseline)]
county.mobility[, moveidx_grocery_pharmacy := as.integer(grocery_and_pharmacy_percent_change_from_baseline)]
county.mobility[, moveidx_parks := as.integer(parks_percent_change_from_baseline)]
county.mobility[, moveidx_transit := as.integer(transit_stations_percent_change_from_baseline)]
county.mobility[, moveidx_workplaces := as.integer(workplaces_percent_change_from_baseline)]
county.mobility[, moveidx_residential := as.integer(residential_percent_change_from_baseline)]
county.mobility <- county.mobility[, .(fips, date, moveidx_retail_recreation, moveidx_grocery_pharmacy,
                                       moveidx_parks, moveidx_transit, moveidx_workplaces, moveidx_residential)]

# 4) County boundaries shape file 

if(!file.exists(file.path(datadir, "downloaded_automatically", "tl_2019_us_county.zip"))) {
  download.file("ftp://ftp2.census.gov/geo/tiger/TIGER2019/COUNTY/tl_2019_us_county.zip", 
                file.path(datadir, "downloaded_automatically", "tl_2019_us_county.zip"), method = "curl")}
unzip(file.path(datadir, "downloaded_automatically", "tl_2019_us_county.zip"))
county.shapefile <- st_read("tl_2019_us_county.shp")

# Also generate a data table with county latitude and longitude
county.coordinates <- data.table(fips = county.shapefile$GEOID, latitude = county.shapefile$INTPTLAT, longitude = county.shapefile$INTPTLON)
county.coordinates[, latitude := as.numeric(gsub("+", "", latitude))][, longitude := as.numeric(gsub("+", "", longitude))]
invisible(lapply(c("tl_2019_us_county.cpg", "tl_2019_us_county.dbf", "tl_2019_us_county.prj", "tl_2019_us_county.shp", 
                   "tl_2019_us_county.shp.ea.iso.xml", "tl_2019_us_county.shp.iso.xml", "tl_2019_us_county.shx"), unlink))

# 5) School district boundaries shape file

if(!file.exists(file.path(datadir, "downloaded_automatically", "EDGE_SCHOOLDISTRICT_TL19_SY1819.zip"))) {
  download.file("https://nces.ed.gov/programs/edge/data/EDGE_SCHOOLDISTRICT_TL19_SY1819.zip", 
                file.path(datadir, "downloaded_automatically", "EDGE_SCHOOLDISTRICT_TL19_SY1819.zip"), method = "curl")}
unzip(file.path(datadir, "downloaded_automatically", "EDGE_SCHOOLDISTRICT_TL19_SY1819.zip"))
district.shapefile <- st_read("schooldistrict_sy1819_tl19.shp")
invisible(lapply(c("schooldistrict_sy1819_tl19.CPG", "schooldistrict_sy1819_tl19.dbf", "schooldistrict_sy1819_tl19.prj", 
                   "schooldistrict_sy1819_tl19.sbn", "schooldistrict_sy1819_tl19.sbx", "schooldistrict_sy1819_tl19.shp", 
                   "schooldistrict_sy1819_tl19.shp.xml", "schooldistrict_sy1819_tl19.shx"), unlink))

# 6) County and state COVID-19 policy ordinances 

if(!file.exists(file.path(datadir, "downloaded_automatically", "state_county_covid19_policies.csv"))) {
  download.file("https://healthdata.gov/api/views/gyqz-9u7n/rows.csv?accessType=DOWNLOAD",
                file.path(datadir, "downloaded_automatically", "state_county_covid19_policies.csv"), method = "curl")}
state.county.ordinances <- fread(file.path(datadir, "downloaded_automatically", "state_county_covid19_policies.csv"))
setnames(state.county.ordinances, "fips_code", "fips")
state.county.ordinances[substr(date, 1, 5) == "0020/", date := gsub("0020/", "2020/", date)]
state.county.ordinances[, date := as.Date(date, format = "%Y/%m/%d")]
state.county.ordinances[date <= "2020-01-01", date := as.Date("2020-01-01")]
state.county.ordinances <- unique(state.county.ordinances[, .(state_id, fips, policy_level, policy_type, date, start_stop)], by = NULL)
state.county.ordinances[, ct := .N, by = .(state_id, fips, policy_level, date, policy_type)]
state.county.ordinances <- state.county.ordinances[ct == 1]

# Separate county-level and state-level policies, and merge state-level policies to the county fips codes
county.ordinances <- state.county.ordinances[policy_level == "county", .(fips, policy_type, date, county_start_stop = start_stop)]
county.ordinances[, fips := sprintf("%05d", fips)]
state.ordinances <- state.county.ordinances[policy_level == "state", .(state_id, policy_type, date, state_start_stop = start_stop)]

temp.cw.stusps.fips <- data.table(
  state_id = c("AL", "AK", "AZ", "AR", "CA", "CO", "CT", "DE", "DC", "FL", "GA", "HI", "ID", "IL", "IN", "IA", "KS", "KY", "LA", 
               "ME", "MD", "MA", "MI", "MN", "MS", "MO", "MT", "NE", "NV", "NH", "NJ", "NM", "NY", "NC", "ND", "OH", "OK", "OR", 
               "PA", "RI", "SC", "SD", "TN", "TX", "UT", "VT", "VA", "WA", "WV", "WI", "WY"),
  state_fips = c("01", "02", "04", "05", "06", "08", "09", "10", "11", "12", "13", "15", "16", "17", "18", "19", "20", "21", "22", 
                 "23", "24", "25", "26", "27", "28", "29", "30", "31", "32", "33", "34", "35", "36", "37", "38", "39", "40", "41", 
                 "42", "44", "45", "46", "47", "48", "49", "50", "51", "53", "54", "55", "56"))

temp.cw.stusps.fips <- merge(temp.cw.stusps.fips, county.coordinates[, .(state_fips = substr(fips, 1, 2), fips)], by = "state_fips")[, .(state_id, fips)]
state.ordinances <- merge(state.ordinances, temp.cw.stusps.fips, by  = "state_id",  allow.cartesian = TRUE)[, .(fips, policy_type, date, state_start_stop)]

# Compile a balanced panel of counties -X- policy type -X- dates
county.ordinances.panel <- CJ(fips = unique(temp.cw.stusps.fips$fips), policy_type = unique(state.county.ordinances$policy_type),
                              date = seq.Date(from = min(state.county.ordinances$date), to = max(county.covid$date), by = "days"))
county.ordinances.panel <- merge(county.ordinances.panel, state.ordinances, by = c("fips", "policy_type", "date"), all.x = TRUE)
county.ordinances.panel <- merge(county.ordinances.panel, county.ordinances, by = c("fips", "policy_type", "date"), all.x = TRUE)
setkey(county.ordinances.panel, fips, policy_type, date)
county.ordinances.panel[state_start_stop == "start", state_ordinance := 1L]
county.ordinances.panel[state_start_stop == "stop", state_ordinance := 0L]
county.ordinances.panel[is.na(state_ordinance) & date == min(date), state_ordinance := 0L]
county.ordinances.panel[, state_ordinance := na.locf(state_ordinance), by = .(fips, policy_type)]
county.ordinances.panel[county_start_stop == "start", county_ordinance := 1L]
county.ordinances.panel[county_start_stop == "stop", county_ordinance := 0L]
county.ordinances.panel[is.na(county_ordinance) & date == min(date), county_ordinance := 0L]
county.ordinances.panel[, county_ordinance := na.locf(county_ordinance), by = .(fips, policy_type)]
county.ordinances.panel <- county.ordinances.panel[, .(fips, date, policy_type, state_ordinance, county_ordinance)]
county.ordinances.panel[, any_ordinance := 0L]
county.ordinances.panel[state_ordinance == 1, any_ordinance := 1L]
county.ordinances.panel[county_ordinance == 1, any_ordinance := 1L]
setkey(county.ordinances.panel, fips, date, policy_type)

# 7) County population estimates 

if(!file.exists(file.path(datadir, "downloaded_automatically", "county-popest-2019.csv"))) {
  download.file("https://www2.census.gov/programs-surveys/popest/datasets/2010-2019/counties/totals/co-est2019-alldata.csv", 
                file.path(datadir, "downloaded_automatically", "county-popest-2019.csv"))}
county.population <- fread(file.path(datadir, "downloaded_automatically", "county-popest-2019.csv"), 
                           select = c("SUMLEV", "STNAME", "CTYNAME", "STATE", "COUNTY", "POPESTIMATE2019"))
county.population[, fips := paste0(sprintf("%02d", STATE), sprintf("%03d", COUNTY))]
setnames(county.population, "STNAME", "state_name")
county.population[, county_name := gsub(" County", "", CTYNAME)]
county.population <- county.population[SUMLEV == 50, .(state_name, county_name, fips, population = POPESTIMATE2019)]

# 8) County Gross Domestic Product

if(!file.exists(file.path(datadir, "downloaded_automatically", "county-GDP.zip"))) {
  download.file("https://apps.bea.gov/regional/zip/CAGDP1.zip", file.path(datadir, "downloaded_automatically", "county-GDP.zip"))}
unzip(file.path(datadir, "downloaded_automatically", "county-GDP.zip"), files = c("CAGDP1__ALL_AREAS_2001_2019.csv"))
county.gdp <- fread("CAGDP1__ALL_AREAS_2001_2019.csv", select = c("GeoFIPS", "LineCode", "2019"), fill = TRUE)
unlink("CAGDP1__ALL_AREAS_2001_2019.csv")
county.gdp <- suppressWarnings(county.gdp[LineCode == 3, .(fips = GeoFIPS, nominal_gdp = as.numeric(`2019`))])
county.gdp <- county.gdp[as.integer(substr(fips, 1, 2)) %in% 1:56 & substr(fips, 3, 5) != "000"]

# 9) County Small Area Income and Poverty Estimates (SAIPE) for 2019

if(!file.exists(file.path(datadir, "downloaded_automatically", "SAIPE-state-county-2019.xls"))) {
  download.file("https://www2.census.gov/programs-surveys/saipe/datasets/2019/2019-state-and-county/est19all.xls", 
                file.path(datadir, "downloaded_automatically", "SAIPE-state-county-2019.xls"))}
county.saipe <- data.table(read_xls(file.path(datadir, "downloaded_automatically", "SAIPE-state-county-2019.xls"), skip = 3))
county.saipe[, fips := paste0(`State FIPS Code`, `County FIPS Code`)]
county.saipe[, poverty_rate := suppressWarnings(as.numeric(`Poverty Percent, All Ages`))]
county.saipe[, median_income := suppressWarnings(as.integer(`Median Household Income`))]
county.saipe <- county.saipe[`County FIPS Code` != "000", .(fips, median_income, poverty_rate)]

#### LOAD THE DATA COMPONENTS THAT MUST BE DOWNLOADED MANUALLY ####

setwd(file.path(datadir, "downloaded_manually"))

# *** NOTE: these below data must be downloaded manually and are loaded but not downloaded by this code ***

# 10) School district policies for the 2020-2021 school year 

district.policies <- data.table(read_xlsx(file.path(datadir, "downloaded_manually", "mchsd-school-district-policy-covid-data-20201102.xlsx")))
setnames(district.policies, "DistrictNCES", "district_nces")
district.policies[, enrollment := as.integer(Enrollment)]
district.policies[, open_date := as.Date(OpenDate, format = "%m/%d/%Y")]
district.policies[TeachingMethod == "Online Only", classroom := "Remote"]
district.policies[TeachingMethod == "Hybrid", classroom := "Hybrid"]
district.policies[TeachingMethod == "On Premises", classroom := "Campus"]
district.policies[TeachingMethod %in% c("Pending", "Unknown", "Other"), classroom := "Unknown"]
district.policies <- district.policies[!is.na(district_nces), .(district_nces, enrollment, open_date, classroom)]

# 11) County COVID-19 test volume

county.covid.tests <- fread("covid_testing_county_date_20200301_20210531.csv")[, .(geo_value, time_value, value)]
setnames(county.covid.tests, c("fips", "date", "daily_test_volume"))
county.covid.tests[, fips := sprintf("%05d", fips)]

# 12) County demographic labor force participation

county.acs.laborforce <- suppressWarnings(
  fread(cmd = paste0("unzip -p ", file.path(datadir, "downloaded_manually", "ACSST5Y2019.S2301_2021-01-06T134341.zip"))))
county.acs.laborforce[, fips := substr(GEO_ID, 10, 14)]
county.acs.laborforce[, adult_lfpr := suppressWarnings(as.numeric(S2301_C02_021E))]
county.acs.laborforce[, female_lfpr := suppressWarnings(as.numeric(S2301_C02_023E))]
county.acs.laborforce[, mothers_lfpr := suppressWarnings(as.numeric(S2301_C02_024E))]
county.acs.laborforce <- county.acs.laborforce[GEO_ID != "id", .(fips, adult_lfpr, female_lfpr, mothers_lfpr)]

# 13) County demographic characteristics

county.demographics <- suppressWarnings(
  fread(cmd = paste0("unzip -p ", file.path(datadir, "downloaded_manually", "ACSDP5Y2019.DP05_2021-01-06T135818.zip"))))
county.demographics[, fips := substr(GEO_ID, 10, 14)]
county.demographics[, pop_u18_pct := suppressWarnings(as.numeric(DP05_0019PE))]
county.demographics <- county.demographics[GEO_ID != "id", .(fips, pop_u18_pct)]

# 14) County 2019 health care characteristics

county.flu.vaccination.rates <- fread(file.path(datadir, "downloaded_manually", "2019_county_influenza_vaccine_rate.csv"))
county.flu.vaccination.rates[, fips := sprintf("%05d", fips)]
county.flu.vaccination.rates[, flu_vaccine_rate := as.integer(analysis_value)]
county.flu.vaccination.rates <- county.flu.vaccination.rates[, .(fips, flu_vaccine_rate)]

# 15) County 2019 total and ICU hospital bed counts

county.hospital.beds <- fread(file.path(datadir, "downloaded_manually", "Updated_HospitalBedsPerCounty.csv"))
county.hospital.beds[, fips := sprintf("%05d", as.integer(GEOID))]
county.hospital.beds[, hospital_total_beds := as.integer(Beds)]
county.hospital.beds[, hospital_icu_beds := as.integer(ICUBeds)]
county.hospital.beds <- county.hospital.beds[, .(fips, hospital_total_beds, hospital_icu_beds)]

#### COMPILE THE PANEL OF COUNTIES ####

setwd(datadir)

# Form the base panel of counties -X- dates: 2020-March-01 to 2021-May-31
# Note: for monthly employment, also including values for 2019-January through 2020-February

county.panel <- CJ(fips = unique(county.population$fips), 
                   date = c(unique(county.employment[ monthdate >= "2019-01-01" & monthdate < "2020-03-01"]$monthdate), 
                            unique(county.covid[date >= "2020-03-01" & date <= "2021-05-31"]$date)))

# Define columns for the weekly "weekdate" and monthly "monthdate" 

county.panel[, temp_weeknum := floor(as.integer(difftime(date, "2020-02-29", units = "days")) / 7)]
county.panel[date >= "2020-03-01", weekdate := min(date), by = temp_weeknum]
county.panel[, monthdate := min(date), by = .(year(date), month(date))]
county.panel$temp_weeknum <- NULL

# Merge to panel the fixed values of county population estimates for 2019, along with state and county names

county.panel <- merge(county.panel, county.population, by = "fips", all.x = TRUE)

# Merge to panel the fixed values of county coordinates (latitude and longitude)

county.panel <- merge(county.panel, county.coordinates, by = "fips", all.x = TRUE)

# Merge to panel the fixed values of 2019 county GDP

county.panel <- merge(county.panel, county.gdp, by = "fips", all.x = TRUE)

# Merge to panel the fixed values of 2019 county ACS demographics

county.panel <- merge(county.panel, county.demographics, by = "fips", all.x = TRUE)

# Merge to panel the fixed values of 2019 county ACS demographic labor force participation

county.panel <- merge(county.panel, county.acs.laborforce, by = "fips", all.x = TRUE)

# Merge to panel the fixed values of 2019 county median income and poverty percent estimates

county.panel <- merge(county.panel, county.saipe, by = "fips", all.x = TRUE)

# Merge to panel the fixed value of 2019 county influenza vaccination rates

county.panel <- merge(county.panel, county.flu.vaccination.rates, by = "fips", all.x = TRUE)

# Merge to panel the fixed values of 2019 county hospital total beds and ICU beds

county.panel <- merge(county.panel, county.hospital.beds, by = "fips", all.x = TRUE)

# Use the district and county boundaries shape files to map fixed initial district policies to the county level, and merge to panel

district.shapefile <- st_make_valid(district.shapefile)
county.shapefile <- st_make_valid(county.shapefile)
cw.counties.districts.idx <- st_intersects(county.shapefile, district.shapefile)

cw.counties.districts.dtl <- lapply(1:length(county.shapefile$GEOID), function(c) {
  data.table(fips = county.shapefile$GEOID[[c]], district_nces = district.shapefile$GEOID[cw.counties.districts.idx[[c]]])
})
cw.counties.districts <- rbindlist(cw.counties.districts.dtl)
county.district.schooling <- merge(cw.counties.districts, district.policies, by = "district_nces")
length(unique(county.district.schooling$fips)) == length(unique(county.district.schooling[classroom != "Unknown" & !is.na(enrollment)]$fips)) # TRUE
county.district.schooling <- county.district.schooling[classroom != "Unknown" & !is.na(enrollment)]
setorder(county.district.schooling, fips, -enrollment, na.last = TRUE)
county.schooling <- county.district.schooling[, .SD[1], by = fips, .SDcols = c("open_date", "classroom")]
county.panel <- merge(county.panel, county.schooling, by = "fips")

# Merge to panel the daily and cumulative COVID cases and deaths

county.panel <- merge(county.panel, county.covid, by = c("fips", "date"), all.x = TRUE)
county.panel[is.na(cumulative_cases), cumulative_cases := 0]
county.panel[is.na(cumulative_deaths), cumulative_deaths := 0]
county.panel[, daily_cases := cumulative_cases - shift(cumulative_cases)]
county.panel[, daily_deaths := cumulative_deaths - shift(cumulative_deaths)]
county.panel[is.na(daily_cases) | daily_cases < 0, daily_cases := 0]
county.panel[is.na(daily_deaths) | daily_deaths < 0, daily_deaths := 0]

# Merge to panel the daily COVID testing volume

county.panel <- merge(county.panel, county.covid.tests, by = c("fips", "date"), all.x = TRUE)

# Merge to panel the monthly employment measures

county.panel <- merge(county.panel, county.employment, by = c("fips", "monthdate"), all.x = TRUE)
county.panel[date != monthdate, c("unemployment_rate", "unemployment", "employment", "labor_force") := NA]
setcolorder(county.panel, c("fips", "date", "weekdate", "monthdate"))

# Merge to panel the daily county community mobility indexes

county.panel <- merge(county.panel, county.mobility, by = c("fips", "date"), all.x = TRUE)

# Merge to panel the daily count of county and state COVID-19 policy ordinances

county.ordinances.panel.counts <- county.ordinances.panel[, .(
  state_covid_ordinances = sum(state_ordinance), county_covid_ordinances = sum(county_ordinance), 
  total_covid_ordinances = sum(any_ordinance)), by = .(fips, date)]
county.panel <- merge(county.panel, county.ordinances.panel.counts, by = c("fips", "date"), all.x = TRUE)

# Clean up workspace

rm(county.population, county.saipe, county.demographics, county.acs.laborforce, county.coordinates, 
   county.flu.vaccination.rates, county.hospital.beds, county.shapefile, district.shapefile, 
   cw.counties.districts.idx, cw.counties.districts.dtl, cw.counties.districts, county.district.schooling, 
   district.policies, county.schooling, county.covid, county.covid.tests, county.employment, county.mobility,
   county.ordinances.panel, county.ordinances.panel.counts, county.gdp, state.county.ordinances,
   state.ordinances, county.ordinances, temp.cw.stusps.fips)

#### SAVE COMPILED DATA FOR ANALYSIS, AS BOTH CSV AND STATA FORMAT (VERSION 13) ####

summary(county.panel[, .(date, monthdate)]) # Check date range

setkey(county.panel, fips, date)

fwrite(county.panel, "county_panel.csv")
write_dta(county.panel, "county_panel.dta", version = 13)

#### END ####
